/*
 * Intrinsics.cpp
 *
 *  Created on: Aug 10, 2015
 *      Author: Jan Vrany
 */

#ifndef LLVM_C_EXT_INTRINSICS_H
#define LLVM_C_EXT_INTRINSICS_H

#include <cstring>

#include <llvm/ADT/ArrayRef.h>
#include <llvm/IR/Intrinsics.h>
#include <llvm/IR/Module.h>
#include <llvm-c-ext/Intrinsics.h>


static const char* const intrinsic_names[] = {
#define GET_INTRINSIC_NAME_TABLE
#include "llvm/IR/Intrinsics.gen"
#undef GET_INTRINSIC_NAME_TABLE
};


LLVMValueRef LLVMGetIntrinsicByName(LLVMModuleRef mod, const char* name, const int count, const LLVMTypeRef* types) {
	llvm::Intrinsic::ID id = llvm::Intrinsic::not_intrinsic;
	llvm::Function* f;

	for (int intrinIdx = 0; intrinIdx < (sizeof(intrinsic_names) / sizeof(const char*)); intrinIdx++) {
		if (strcmp(name, intrinsic_names[intrinIdx]) == 0) {
			llvm::Type* typesUnwrapped[count];
			for (int typeIdx = 0; typeIdx < count; typeIdx++) {
				typesUnwrapped[typeIdx] = llvm::unwrap(types[typeIdx]);
			}
			llvm::ArrayRef<llvm::Type*> typesArray(typesUnwrapped, count);
			f = llvm::Intrinsic::getDeclaration(llvm::unwrap(mod), (llvm::Intrinsic::ID)(intrinIdx + 1), typesArray);
			return llvm::wrap(f);
		}
	}
	return NULL;
}

#endif /* !defined(LLVM_C_EXT_INTRINSICS_H) */








